@@ -22,6 +22,7 @@ import paramiko
 from tqdm import tqdm
 import scp as scplib
 import humanfriendly
+import pandas as pd
 
 from pyxllib.algo.pupil import natural_sort
 from pyxllib.file.specialist import XlPath
@@ -93,21 +94,33 @@ class XlSSHClient(paramiko.SSHClient):
     def __1_初始化和执行(self):
         pass
 
-    def __init__(self, server, port, user, passwd, *, map_path=None):
+    def __init__(self, server, port, user, passwd, *, map_path=None,
+                 relogin=0, relogin_interval=1):
         """
-        Args:
-            server:
-            port:
-            user:
-            passwd:
-            map_path: 主要在上传、下载文件的时候，可以用来自动定位路径
-                参考写法：{'D:/': '/'}  # 将D盘映射到服务器位置
+
+        :param map_path: 主要在上传、下载文件的时候，可以用来自动定位路径
+            参考写法：{'D:/': '/'}  # 将D盘映射到服务器位置
+        :param int relogin: 当连接失败的时候，可能只是某种特殊原因，可以尝试重新连接
+            该参数设置重连的次数
+        :param int|float relogin_interval: 每次重连的间隔秒数
+
         """
 
         super().__init__()
         self.load_system_host_keys()
         self.set_missing_host_key_policy(paramiko.AutoAddPolicy())
-        self.connect(server, port, user, passwd)
+
+        # 重试登录
+        for k in range(relogin + 1):
+            try:
+                self.connect(server, port, user, passwd)
+                break
+            except paramiko.ssh_exception.SSHException:
+                if k < relogin:
+                    time.sleep(relogin_interval)
+        else:
+            raise paramiko.ssh_exception.SSHException
+
         self.map_path = map_path
 
         class Path(pathlib.PurePosixPath):
@@ -170,75 +183,24 @@ class XlSSHClient(paramiko.SSHClient):
         self.Path = Path
 
     @classmethod
-    def log_in(cls, host_name, user_name, *, relogin=0, relogin_interval=1, map_path=None):
-        r""" 使用XlSshAccounts里存储的服务器、账号信息，进行登录
-        推荐的XlSSHClient初始化方法，可以隐藏IP、账号密码
-
-        :param int relogin: 当连接失败的时候，可能只是某种特殊原因，可以尝试重新连接
-            该参数设置重连的次数
-        :param int|float relogin_interval: 每次重连的间隔秒数
-
-        使用该功能，需要提前存储如下格式的环境变量
-        def 存储服务器信息():
-            import textwrap
-            from pyxllib.prog.specialist import XlOsEnv
-
-            xl_ssh_accounts = textwrap.dedent('''\
-            xlpr0 172.16.170.110:22
-                root,123456
-            titan1 172.16.170.119:22, g_titan1 172.16.170.120:6001
-                root,123456
-                ckz,654321
-            ''')
-
-            XlOsEnv.persist_set('XlSshAccounts', xl_ssh_accounts, encoding=True)
-
-        存写格式说明：
-        xlpr0 172.16.170.110:22   # 设置一个主机昵称 ip:port
-            root,123456 # tab或空格缩进，每行填写一个账号名、账号密码
-        titan1 172.16.170.119:22, g_titan1 172.16.170.120:6001  # 可以逗号隔开设置多台主机，常用于同一个机器的局域网、公网不同连接
-            root,123456
-            ckz,654321  # 可以写多个账号
-
+    def log_in(cls, host_name, user_name, **kwargs):
+        r""" 使用XlprDb里存储的服务器、账号信息，进行登录
         """
-        # 1 找主机信息
-        # 获得的是一个格式化的字符串，还没有做结构化解析
-        xl_ssh_accounts = XlOsEnv.get('XlSshAccounts', decoding=True)
-        lines = xl_ssh_accounts.splitlines()
-        idx, ip_port = None, None
-        for i, line in enumerate(lines):
-            if not re.match(r'\s', line):  # 服务器信息
-                hosts = line.split(',')
-                for host in hosts:
-                    m = re.match(rf'{host_name}\s+(.+)', host.strip())
-                    if m:
-                        idx, ip_port = i, m.group(1)
-                        break
-                if ip_port is not None:
-                    break
+        from pyxllib.data.pglib import connect_xlprdb
 
-        if idx is None:
-            raise ValueError(f'host_name={host_name} not found!')
-        host_ip, host_port = ip_port.split(':')
-
-        # 2 找账户信息
-        for i in range(idx + 1, len(lines)):
-            if re.match(r'\s', lines[i]):
-                # 一般应该没人会在密码以空格为后缀，所以这里使用strip，排除右边误输入空格的情况
-                # 更人性化，但是不严谨，遇到空格后缀的密码本套功能会处理不了
-                name, passwd = lines[i].strip().split(',')
-                if name == user_name:
-                    for k in range(relogin + 1):
-                        try:
-                            ssh = cls(host_ip, host_port, name, passwd, map_path=map_path)
-                            return ssh
-                        except paramiko.ssh_exception.SSHException:
-                            if k < relogin:
-                                time.sleep(relogin_interval)
+        with connect_xlprdb() as con:
+            if host_name.startswith('g_'):
+                host_ip = con.execute("SELECT host_ip FROM hosts WHERE host_name='xlpr0'").fetchone()[0]
+                pw, port = con.execute('SELECT (pgp_sym_decrypt(accounts, %s)::jsonb)[%s]::text, frpc_port'
+                                       ' FROM hosts WHERE host_name=%s',
+                                       (con.seckey, user_name, host_name[2:])).fetchone()
             else:
-                break
+                port = 22
+                host_ip, pw = con.execute('SELECT host_ip, (pgp_sym_decrypt(accounts, %s)::jsonb)[%s]::text'
+                                          ' FROM hosts WHERE host_name=%s',
+                                          (con.seckey, user_name, host_name)).fetchone()
 
-        raise ValueError(f'user_name={user_name} not found!')
+        return cls(host_ip, port, user_name, pw[1:-1], **kwargs)
 
     def exec(self, command, *args, **kwargs):
         """ exec_command的简化版
@@ -545,39 +507,75 @@ class XlSSHClient(paramiko.SSHClient):
         if sudo:
             self.exec(f'usermod -aG sudo {name}')
 
+    def check_cpu_usage(self, *, print_mode=False):
+        # 1 获取原始信息
+        cmds = ['ps --no-headers -eo "pcpu,pmem,user"',  # 列出所有程序情况
+                # 使用awk，按用户分组统计cpu（百分比）、内存总使用量（绝对量）
+                """awk 'BEGIN{FS=OFS=" "}{a0[$3]+=$1; a1[$3]+=$2}END {for (i in a0) print i,a0[i],a1[i]}'""",
+                'sort -rn -k1,2 -k2,3'  # 按cpu、内存使用量从大到小排序，不是必须的
+                ]
+        lines = self.exec('|'.join(cmds)).splitlines()
+
+        # 2 按用户分组统计
+        def f(v):
+            return round(float(v), 2)
+
+        user_usage = {}
+        for line in lines:
+            user, cpu, mem = line.split()
+            cpu, mem = f(cpu), f(mem)
+            if cpu + mem > 0.0:
+                user_usage[user] = [cpu, mem]
+        _total = [f(sum([v[0] for v in user_usage.values()])), f(sum([v[1] for v in user_usage.values()]))]
+        user_usage['_total'] = _total
+
+        # 3 输出
+        if print_mode:
+            df = pd.DataFrame.from_records([[k, v[0], v[1]] for k, v in user_usage.items()],
+                                           columns=['user', '%cpu', '%mem'])
+            print(df)
+
+        return user_usage
+
     def check_gpu_usage(self, *, print_mode=False):
         """ 检查(每个用户)显存使用量
 
         使用这个命令，必须要求远程服务器安装了pip install gpustat
+
+        TODO 加上记录最高显卡温度？
         """
+        # 1 获取原始信息
+        lines = self.exec('gpustat').splitlines()[1:]
+        if print_mode:
+            print('\n'.join(lines))
 
-        def printf(*args, **kwargs):
-            if print_mode:
-                print(*args, **kwargs)
-
-        ls = self.exec('gpustat')
-        printf(ls)
-
-        total_memory = 0
+        # 2 按用户分组统计
         user_usage = defaultdict(float)
-        for line in ls.splitlines()[1:]:
+        for line in lines:
             name, temperature, capacity, uses = line.split('|')
             used, total = map(int, re.findall(r'\d+', capacity))
-            user_usage['other'] += used / 1024
-            total_memory += total / 1024
+            user_usage['_other'] += used / 1024  # 会有些没有用户操作的僵尸显存占用
             for x in uses.split():
-                user, used = re.search(r'(.+?)\((\d+)M\)', x).groups()
-                user_usage[user] += int(used) / 1024
-                user_usage['other'] -= int(used) / 1024
+                user, one_used = re.search(r'(.+?)\((\d+)M\)', x).groups()
+                one_used = int(one_used) / 1024
+                user_usage[user] += one_used
+                user_usage['_other'] -= one_used
 
         # 使用量从多到少排序。但注意如果转存到PG，jsonb会重新排序。
         user_usage = {k: round(user_usage[k], 2) for k in sorted(user_usage, key=lambda k: -user_usage[k])}
-        if user_usage['other'] < 0.01:
-            del user_usage['other']
+        if user_usage['_other'] < 0.01:
+            del user_usage['_other']
 
-        return total_memory, user_usage
+        user_usage['_total'] = round(sum(user_usage.values()), 2)
 
-    def check_disk_usage(self):
+        # 3 输出
+        if print_mode:
+            df = pd.DataFrame.from_records([[k, v] for k, v in user_usage.items()], columns=['user', 'gpu_mem_GB'])
+            print(df)
+
+        return user_usage
+
+    def check_disk_usage(self, *, print_mode=False, timeout=1200):
         """ 检查(每个用户)磁盘空间使用量
 
         :return: total 总字节MBytes数, msg 所有用户、包括其他非/home目录的使用MBytes数
@@ -595,13 +593,20 @@ class XlSSHClient(paramiko.SSHClient):
 
         # 2 /home目录下每个占用情况
         user_usage = defaultdict(int)
-        for line in self.exec('du -d 1 /home').splitlines():
+        for line in self.exec('du -d 1 /home', timeout=timeout).splitlines():  # 这个要限时，默认20分钟
             _bytes, _dir = line.split(maxsplit=1)
             _dir = _dir[6:]
             if _dir and int(_bytes) > GB:  # 达到1GB的才记录
                 user_usage[_dir] += int(_bytes)
 
-        user_usage['other'] = used - sum(user_usage.values())
+        user_usage['_other'] = used - sum(user_usage.values())
         user_usage = {k: (v // GB) for k, v in user_usage.items()}
+        user_usage = {k: round(user_usage[k], 2) for k in sorted(user_usage, key=lambda k: -user_usage[k])}
+        user_usage['_total'] = total_memory // GB
 
-        return total_memory // GB, user_usage
+        # 3 展示
+        if print_mode:
+            df = pd.DataFrame.from_records([[k, v] for k, v in user_usage.items()], columns=['user', 'disk_mem_GB'])
+            print(df)
+
+        return user_usage
